#ifndef GITDIFF_H
#define GITDIFF_H
// generated from class_header.h
#include <nan.h>
#include <string>

extern "C" {
#include <git2.h>
#include <git2/sys/diff.h>
}

#include "../include/diff.h"
#include "../include/diff_find_options.h"
#include "../include/diff_delta.h"
#include "../include/diff_perfdata.h"
#include "../include/repository.h"
#include "../include/index.h"
#include "../include/diff_options.h"
#include "../include/tree.h"
// Forward declaration.
struct git_diff {
};

using namespace node;
using namespace v8;

class GitDiff : public ObjectWrap {
public:

  static Persistent<Function> constructor_template;
  static void InitializeComponent (Handle<v8::Object> target);

  git_diff *GetValue();
  git_diff **GetRefValue();
  void ClearValue();

  static Handle<v8::Value> New(void *raw, bool selfFreeing);
  bool selfFreeing;



private:


  GitDiff(git_diff *raw, bool selfFreeing);
  ~GitDiff();

  static NAN_METHOD(JSNewFunction);


  struct FindSimilarBaton {
    int error_code;
    const git_error* error;
    git_diff * diff;
    const git_diff_find_options * options;
  };
  class FindSimilarWorker : public NanAsyncWorker {
  public:
    FindSimilarWorker(
      FindSimilarBaton *_baton,
      NanCallback *callback
    ) : NanAsyncWorker(callback)
      , baton(_baton) {};
    ~FindSimilarWorker() {};
    void Execute();
    void HandleOKCallback();

  private:
    FindSimilarBaton *baton;
  };

  static NAN_METHOD(FindSimilar);

  static NAN_METHOD(GetDelta);

  struct GetPerfdataBaton {
    int error_code;
    const git_error* error;
    git_diff_perfdata * out;
    const git_diff * diff;
  };
  class GetPerfdataWorker : public NanAsyncWorker {
  public:
    GetPerfdataWorker(
      GetPerfdataBaton *_baton,
      NanCallback *callback
    ) : NanAsyncWorker(callback)
      , baton(_baton) {};
    ~GetPerfdataWorker() {};
    void Execute();
    void HandleOKCallback();

  private:
    GetPerfdataBaton *baton;
  };

  static NAN_METHOD(GetPerfdata);

  struct IndexToWorkdirBaton {
    int error_code;
    const git_error* error;
    git_diff * diff;
    git_repository * repo;
    git_index * index;
    const git_diff_options * opts;
  };
  class IndexToWorkdirWorker : public NanAsyncWorker {
  public:
    IndexToWorkdirWorker(
      IndexToWorkdirBaton *_baton,
      NanCallback *callback
    ) : NanAsyncWorker(callback)
      , baton(_baton) {};
    ~IndexToWorkdirWorker() {};
    void Execute();
    void HandleOKCallback();

  private:
    IndexToWorkdirBaton *baton;
  };

  static NAN_METHOD(IndexToWorkdir);

  static NAN_METHOD(NumDeltas);

  struct TreeToIndexBaton {
    int error_code;
    const git_error* error;
    git_diff * diff;
    git_repository * repo;
    git_tree * old_tree;
    git_index * index;
    const git_diff_options * opts;
  };
  class TreeToIndexWorker : public NanAsyncWorker {
  public:
    TreeToIndexWorker(
      TreeToIndexBaton *_baton,
      NanCallback *callback
    ) : NanAsyncWorker(callback)
      , baton(_baton) {};
    ~TreeToIndexWorker() {};
    void Execute();
    void HandleOKCallback();

  private:
    TreeToIndexBaton *baton;
  };

  static NAN_METHOD(TreeToIndex);

  struct TreeToTreeBaton {
    int error_code;
    const git_error* error;
    git_diff * diff;
    git_repository * repo;
    git_tree * old_tree;
    git_tree * new_tree;
    const git_diff_options * opts;
  };
  class TreeToTreeWorker : public NanAsyncWorker {
  public:
    TreeToTreeWorker(
      TreeToTreeBaton *_baton,
      NanCallback *callback
    ) : NanAsyncWorker(callback)
      , baton(_baton) {};
    ~TreeToTreeWorker() {};
    void Execute();
    void HandleOKCallback();

  private:
    TreeToTreeBaton *baton;
  };

  static NAN_METHOD(TreeToTree);

  struct TreeToWorkdirBaton {
    int error_code;
    const git_error* error;
    git_diff * diff;
    git_repository * repo;
    git_tree * old_tree;
    const git_diff_options * opts;
  };
  class TreeToWorkdirWorker : public NanAsyncWorker {
  public:
    TreeToWorkdirWorker(
      TreeToWorkdirBaton *_baton,
      NanCallback *callback
    ) : NanAsyncWorker(callback)
      , baton(_baton) {};
    ~TreeToWorkdirWorker() {};
    void Execute();
    void HandleOKCallback();

  private:
    TreeToWorkdirBaton *baton;
  };

  static NAN_METHOD(TreeToWorkdir);

  struct TreeToWorkdirWithIndexBaton {
    int error_code;
    const git_error* error;
    git_diff * diff;
    git_repository * repo;
    git_tree * old_tree;
    const git_diff_options * opts;
  };
  class TreeToWorkdirWithIndexWorker : public NanAsyncWorker {
  public:
    TreeToWorkdirWithIndexWorker(
      TreeToWorkdirWithIndexBaton *_baton,
      NanCallback *callback
    ) : NanAsyncWorker(callback)
      , baton(_baton) {};
    ~TreeToWorkdirWithIndexWorker() {};
    void Execute();
    void HandleOKCallback();

  private:
    TreeToWorkdirWithIndexBaton *baton;
  };

  static NAN_METHOD(TreeToWorkdirWithIndex);
  git_diff *raw;
};

#endif
